"""Beads (bd) gateway for issue operations.

This package provides an abstract interface and implementations for Beads issue operations.

Import from submodules:
- erk_shared.gateway.beads.abc: BeadsGateway (ABC)
- erk_shared.gateway.beads.real: RealBeadsGateway
- erk_shared.gateway.beads.fake: FakeBeadsGateway
- erk_shared.gateway.beads.dry_run: DryRunBeadsGateway
- erk_shared.gateway.beads.printing: PrintingBeadsGateway
- erk_shared.gateway.beads.types: BeadsIssue
"""
