"""Status information collectors.

Import from submodules:
- base: StatusCollector
- git: GitStatusCollector
- github: GitHubPRCollector
- graphite: GraphiteStackCollector
- impl: PlanFileCollector
"""
