"""Repository operations sub-gateway.

This module provides a separate gateway for repository location operations.

Import from submodules:
- abc: GitRepoOps
- real: RealGitRepoOps
- fake: FakeGitRepoOps
- dry_run: DryRunGitRepoOps
- printing: PrintingGitRepoOps
"""
