"""Analysis operations sub-gateway.

This module provides a separate gateway for branch analysis operations.

Import from submodules:
- abc: GitAnalysisOps
- real: RealGitAnalysisOps
- fake: FakeGitAnalysisOps
- dry_run: DryRunGitAnalysisOps
- printing: PrintingGitAnalysisOps
"""
