"""GitHub admin operations for erk-queue workflow.

Import from submodules:
- abc: GitHubAdmin
- real: RealGitHubAdmin
- fake: FakeGitHubAdmin
- noop: NoopGitHubAdmin
- printing: PrintingGitHubAdmin
"""
