"""Codespace registry gateway for managing registered GitHub Codespaces.

Import from submodules:
- abc: CodespaceRegistry, RegisteredCodespace
- real: RealCodespaceRegistry, register_codespace, unregister_codespace, set_default_codespace
- fake: FakeCodespaceRegistry
"""
