"""Tests for erk launch pr-fix-conflicts command.

Note: The old `erk pr fix-conflicts remote` command has been replaced by
`erk launch pr-fix-conflicts`. See tests/commands/launch/test_launch_cmd.py
for the new workflow tests.
"""

# This file is kept for reference. The tests have been migrated to:
# - tests/commands/launch/test_launch_cmd.py
#
# The `erk pr fix-conflicts remote` subcommand has been removed.
# Use `erk launch pr-fix-conflicts` instead.
