"""Configuration operations sub-gateway.

This module provides a separate gateway for git configuration operations.

Import from submodules:
- abc: GitConfigOps
- real: RealGitConfigOps
- fake: FakeGitConfigOps
- dry_run: DryRunGitConfigOps
- printing: PrintingGitConfigOps
"""
