"""Fake GitHub Actions admin operations for testing.

Re-exports FakeGitHubAdmin from erk_shared for backwards compatibility.
"""

from erk_shared.gateway.github_admin.fake import FakeGitHubAdmin as FakeGitHubAdmin
