"""GitHub integration utilities.

Import from submodules:
- abc: GitHub
- types: PRInfo, PRMergeability, PRState, PullRequestInfo, WorkflowRun
- metadata_blocks: PLAN_METADATA_MARKER_END, PLAN_METADATA_MARKER_START, etc.
- parsing: _parse_github_pr_url
- issues: GitHubIssues, RealGitHubIssues, FakeGitHubIssues, etc.
- real: RealGitHub
"""
