"""Status command implementation.

Import from submodules:
- orchestrator: StatusOrchestrator
- models.status_data: StatusData, GitStatus, etc.
- collectors: StatusCollector, GitStatusCollector, etc.
- renderers.simple: SimpleRenderer
"""
