.action-toolbar {
  display: flex;
  align-items: center;
  gap: 6px;
  padding: 6px 10px;
  background: #2d2d2d;
  border-bottom: 1px solid #404040;
  flex-shrink: 0;
}

.action-toolbar__button {
  padding: 4px 12px;
  border: 1px solid #555;
  border-radius: 3px;
  background: #3c3c3c;
  color: #d4d4d4;
  font-family: "Menlo", "Consolas", "Courier New", monospace;
  font-size: 11px;
  cursor: pointer;
  white-space: nowrap;
  user-select: none;
}

.action-toolbar__button:hover:not(:disabled) {
  background: #4a4a4a;
  border-color: #888;
}

.action-toolbar__button:disabled {
  opacity: 0.4;
  cursor: default;
  pointer-events: none;
}

.action-toolbar__button--running {
  opacity: 0.6;
  cursor: wait;
}
